/*
	Applet that repeatedly displays a memory image - 9/97 O. Lawlor
*/

import java.applet.Applet;
import java.awt.*;
import java.awt.image.*;
import ColorMap;
import DerampModel;

class AnimWindow extends Panel {
	byte [] pix=null;
	int w,h;
	ColorMap cm=null;
/*Size:*/
	public Dimension getPreferredSize() {return preferredSize();}
	public Dimension preferredSize() {return new Dimension(w,h);}
	public Dimension getMinimumSize() {return preferredSize();}
	public Dimension minimumSize() {return preferredSize();}
	public Dimension getMaximumSize() {return preferredSize();}
	public Dimension maximumSize() {return preferredSize();}
/*Maintainance:*/
	public void setPix(byte [] Npix,int Nw,int Nh)
	{
		pix=Npix;
		w=Nw;h=Nh;
	}
	public void setMap(ColorMap Ncm)
	{
		cm=Ncm;
	}
/*Update:*/
/*This is key to eliminate flashing.*/
	public void update(Graphics g) {
		paint(g);
	}
	synchronized public void paint(Graphics g) 
	{
		Image img = createImage(new MemoryImageSource(w,h,
				cm.getColorModel(),pix,0,w));
		g.drawImage(img, 0, 0, null);
	}
}

public class DerampApplet extends Applet
{
	
	byte pix[]=null;
	int w=256,h=256;
	ColorMap cm;
	AnimWindow anim;
	Choice color;
	Choice sat;
	TextField norm,norm_delt,para,para_delt;
	DerampModel model;
	
	GridBagConstraints c=new GridBagConstraints();
	GridBagLayout layout=new GridBagLayout();
	void addComp(int x,int y,Component comp)
	{
		c.gridx=x;c.gridy=y;
		layout.setConstraints(comp,c);
		add(comp);
	}
	boolean in_init=true;
/*Init: Lays out all user interface widgets.*/
	public void init() {
	/*Allocate stuff.*/
		pix=new byte[w*h];
		cm=new ColorMap();
		anim=new AnimWindow();
		model=new DerampModel();
		
	/*Set up our Layout.*/
		setLayout(layout);
		System.out.println("Layout set.");
		c.weightx = 1.0;c.weighty = 1.0;
		c.insets=new Insets(2,2,2,2);
		
	/*Labels:*/
		c.gridwidth=3;
		addComp(0,0,new Label("Interactive Flat-Earth Correction"));
		
		c.anchor=GridBagConstraints.WEST;
		c.gridwidth=2;
		addComp(0,1,new Label("Baseline:"));
		
		c.gridwidth=1;
	
	/*Normal baseline controls.*/
		addComp(0,2,new Label("Normal"));
		addComp(0,3,norm=new TextField("0.0",6));
		
		addComp(1,2,new Label("Delta"));
		addComp(1,3,norm_delt=new TextField("0.0",6));
	
	/*Parallel baseline controls.*/
		addComp(0,4,new Label("Parallel"));
		addComp(0,5,para=new TextField("0.0",6));
		
		addComp(1,4,new Label("Delta"));
		addComp(1,5,para_delt=new TextField("0.0",6));
	
	/*Spacer*/
		addComp(0,6,new Label(" "));
		
	/*Satellite control*/
	
		addComp(0,7,new Label("Satellite"));
		
		sat=new Choice();
		sat.addItem("ERS-1/2");
		sat.addItem("JERS");
		sat.addItem("RADARSAT ST 2");
		sat.addItem("RADARSAT FN 2");
		update_sat();
		addComp(1,7,sat);
	
	/*Display control*/
		addComp(0,8,new Label("Display"));
		
		
		color=new Choice();
		color.addItem("Grey");
		color.addItem("Color");
		update_color();
		addComp(1,8,color);
	
		
	/*Display*/
		c.weightx=3.0;
		c.gridheight=20;
		anim.setPix(pix,w,h);
		addComp(2,1,anim);
		
		
	/*Cleanup:*/
		in_init=false;
	}
	
	public static void main(String args[]) {
		DerampApplet app=new DerampApplet();
		app.init();
	}
	
	public boolean action(Event ev,Object obj)
	{
		if (ev.id==Event.ACTION_EVENT)
		{
			if (ev.target instanceof TextField)
				update_base();
			else if (ev.target instanceof Button)
				update_anim();
			else if (ev.target==sat)
				update_sat();
			else if (ev.target==color)
				update_color();
			return true;
		}
		return false;
	}
	void update_base()
	{
		try {
		model.setBaseline(
			(Double.valueOf(norm     .getText())).doubleValue(),
			(Double.valueOf(norm_delt.getText())).doubleValue(),
			(Double.valueOf(para     .getText())).doubleValue(),
			(Double.valueOf(para_delt.getText())).doubleValue());
		} catch (NumberFormatException e)
		{
			/*This is bad.*/
		}
		update_anim();
	}
	void update_sat()
	{
		model.setSatellite(sat.getSelectedIndex());
		update_anim();
	}
	void update_color()
	{
		if (color.getSelectedIndex()==1)
		{/*Color*/
			int [] dex={0  ,40 ,60 ,120,160,200,255};
			int [] red={255,255,255,0  ,0  ,0  ,255};
			int [] grn={0  ,0  ,255,255,255,0  ,0  };
			int [] blu={255,0  ,0  ,0  ,255,255,255};
			cm.addBreaks(dex,red,grn,blu);
		}
		else /*GreyScale*/
			cm.initGrey();
		anim.setMap(cm);
		update_anim();
	}
	void update_anim()
	{
		if (!in_init)
		{
			model.fillBuffer(pix,w,h);
			anim.setPix(pix,w,h);
			anim.repaint(30);
		}
	}
}
